@@ -126,7 +126,7 @@ class KanesMethod(object):
             kd_eqs = [dynamicsymbols('dummy_kd')]
 
         if not isinstance(frame, ReferenceFrame):
-            raise TypeError('An intertial ReferenceFrame must be supplied')
+            raise TypeError('An inertial ReferenceFrame must be supplied')
         self._inertial = frame
 
         self._fr = None
@@ -271,7 +271,7 @@ class KanesMethod(object):
 
     def _form_fr(self, fl):
         """Form the generalized active force."""
-        if fl != None and (len(fl) == 0 or not iterable(fl)):
+        if fl is not None and (len(fl) == 0 or not iterable(fl)):
             raise ValueError('Force pairs must be supplied in an '
                 'non-empty iterable or None.')
 
@@ -279,6 +279,7 @@ class KanesMethod(object):
         # pull out relevant velocities for constructing partial velocities
         vel_list, f_list = _f_list_parser(fl, N)
         vel_list = [msubs(i, self._qdot_u_map) for i in vel_list]
+        f_list = [msubs(i, self._qdot_u_map) for i in f_list]
 
         # Fill Fr with dot product of partial velocities and forces
         o = len(self.u)
@@ -524,7 +525,7 @@ class KanesMethod(object):
             Must be either a non-empty iterable of tuples or None which corresponds
             to a system with no constraints.
         """
-        if (bodies is None and loads != None) or isinstance(bodies[0], tuple):
+        if (bodies is None and loads is not None) or isinstance(bodies[0], tuple):
             # This switches the order if they use the old way.
             bodies, loads = loads, bodies
             SymPyDeprecationWarning(value='The API for kanes_equations() has changed such '
