@@ -49,7 +49,7 @@ from sympy.polys.monomials import (monomial_min, monomial_mul, monomial_div,
 from mpmath.libmp.libintmath import ifac
 from sympy.core import PoleError, Function, Expr
 from sympy.core.numbers import Rational, igcd
-from sympy.core.compatibility import as_int, range
+from sympy.core.compatibility import as_int, range, string_types
 from sympy.functions import sin, cos, tan, atan, exp, atanh, tanh, log, ceiling
 from mpmath.libmp.libintmath import giant_steps
 import math
@@ -170,7 +170,7 @@ def rs_puiseux(f, p, x, prec):
             num, den = power.as_numer_denom()
             n = int(n*den // igcd(n, den))
         elif power != int(power):
-            num, den = power.numerator, power.denominator
+            den = power.denominator
             n = int(n*den // igcd(n, den))
     if n != 1:
         p1 = pow_xin(p, index, n)
@@ -198,7 +198,7 @@ def rs_puiseux2(f, p, q, x, prec):
             num, den = power.as_numer_denom()
             n = n*den // igcd(n, den)
         elif power != int(power):
-            num, den = power.numerator, power.denominator
+            den = power.denominator
             n = n*den // igcd(n, den)
     if n != 1:
         p1 = pow_xin(p, index, n)
@@ -316,7 +316,6 @@ def rs_pow(p1, n, x, prec):
     6*x**2 + 4*x + 1
     """
     R = p1.ring
-    p = R.zero
     if isinstance(n, Rational):
         np = int(n.p)
         nq = int(n.q)
@@ -380,10 +379,10 @@ def rs_subs(p, rules, x, prec):
 
     Parameters
     ----------
-    p : :class:`PolyElement` Input series.
-    rules : :class:`dict` with substitution mappings.
-    x : :class:`PolyElement` in which the series truncation is to be done.
-    prec : :class:`Integer` order of the series after truncation.
+    p : :class:`~.PolyElement` Input series.
+    rules : ``dict`` with substitution mappings.
+    x : :class:`~.PolyElement` in which the series truncation is to be done.
+    prec : :class:`~.Integer` order of the series after truncation.
 
     Examples
     ========
@@ -457,7 +456,6 @@ def _get_constant_term(p, x):
     generators.
     """
     R = p.ring
-    zm = R.zero_monom
     i = R.gens.index(x)
     zm = R.zero_monom
     a = [0]*R.ngens
@@ -562,7 +560,7 @@ def rs_series_inversion(p, x, prec):
     return r
 
 def _coefficient_t(p, t):
-    r"""Coefficient of `x\_i**j` in p, where ``t`` = (i, j)"""
+    r"""Coefficient of `x_i**j` in p, where ``t`` = (i, j)"""
     i, j = t
     R = p.ring
     expv1 = [0]*R.ngens
@@ -578,33 +576,40 @@ def rs_series_reversion(p, x, n, y):
     r"""
     Reversion of a series.
 
-    ``p`` is a series with ``O(x**n)`` of the form `p = a*x + f(x)`
-    where `a` is a number different from 0.
+    ``p`` is a series with ``O(x**n)`` of the form $p = ax + f(x)$
+    where $a$ is a number different from 0.
 
-    `f(x) = sum( a\_k*x\_k, k in range(2, n))`
+    $f(x) = \sum_{k=2}^{n-1} a_kx_k$
+
+    Parameters
+    ==========
 
       a_k : Can depend polynomially on other variables, not indicated.
       x : Variable with name x.
       y : Variable with name y.
 
-    Solve `p = y`, that is, given `a*x + f(x) - y = 0`,
-    find the solution x = r(y) up to O(y**n)
+    Returns
+    =======
 
-    Algorithm:
+    Solve $p = y$, that is, given $ax + f(x) - y = 0$,
+    find the solution $x = r(y)$ up to $O(y^n)$.
 
-    If `r\_i` is the solution at order i, then:
-    `a*r\_i + f(r\_i) - y = O(y**(i + 1))`
+    Algorithm
+    =========
 
-    and if r_(i + 1) is the solution at order i + 1, then:
-    `a*r\_(i + 1) + f(r\_(i + 1)) - y = O(y**(i + 2))`
+    If $r_i$ is the solution at order $i$, then:
+    $ar_i + f(r_i) - y = O\left(y^{i + 1}\right)$
 
-    We have, r_(i + 1) = r_i + e, such that,
-    `a*e + f(r\_i) = O(y**(i + 2))`
-    or `e = -f(r\_i)/a`
+    and if $r_{i + 1}$ is the solution at order $i + 1$, then:
+    $ar_{i + 1} + f(r_{i + 1}) - y = O\left(y^{i + 2}\right)$
+
+    We have, $r_{i + 1} = r_i + e$, such that,
+    $ae + f(r_i) = O\left(y^{i + 2}\right)$
+    or $e = -f(r_i)/a$
 
     So we use the recursion relation:
-    `r\_(i + 1) = r\_i - f(r\_i)/a`
-    with the boundary condition: `r\_1 = y`
+    $r_{i + 1} = r_i - f(r_i)/a$
+    with the boundary condition: $r_1 = y$
 
     Examples
     ========
@@ -666,10 +671,14 @@ def rs_series_from_list(p, c, x, prec, concur=1):
     >>> rs_trunc(pc.compose(x, p), x, 4)
     6*x**3 + 11*x**2 + 8*x + 6
 
+    """
+
+    # TODO: Add this when it is documented in Sphinx
+    """
     See Also
     ========
 
-    sympy.polys.ring.compose
+    sympy.polys.rings.PolyRing.compose
 
     """
     R = p.ring
@@ -686,7 +695,6 @@ def rs_series_from_list(p, c, x, prec, concur=1):
     if r:
         K += 1
     ax = [R(1)]
-    b = 1
     q = R(1)
     if len(p) < 20:
         for i in range(1, J):
@@ -730,8 +738,9 @@ def rs_diff(p, x):
     Return partial derivative of ``p`` with respect to ``x``.
 
     Parameters
-    ----------
-    x : :class:`PolyElement` with respect to which ``p`` is differentiated.
+    ==========
+
+    x : :class:`~.PolyElement` with respect to which ``p`` is differentiated.
 
     Examples
     ========
@@ -761,8 +770,9 @@ def rs_integrate(p, x):
     Integrate ``p`` with respect to ``x``.
 
     Parameters
-    ----------
-    x : :class:`PolyElement` with respect to which ``p`` is integrated.
+    ==========
+
+    x : :class:`~.PolyElement` with respect to which ``p`` is integrated.
 
     Examples
     ========
@@ -801,10 +811,11 @@ def rs_fun(p, f, *args):
         desired series
 
     Parameters
-    ----------
-    p : :class:`PolyElement` The multivariate series to be expanded.
+    ==========
+
+    p : :class:`~.PolyElement` The multivariate series to be expanded.
     f : `ring\_series` function to be applied on `p`.
-    args[-2] : :class:`PolyElement` with respect to which, the series is to be expanded.
+    args[-2] : :class:`~.PolyElement` with respect to which, the series is to be expanded.
     args[-1] : Required order of the expanded series.
 
     Examples
@@ -831,7 +842,7 @@ def rs_fun(p, f, *args):
     else:
         x1 = _x
         p1 = p
-    if isinstance(f, str):
+    if isinstance(f, string_types):
         q = getattr(x1, f)(*args1)
     else:
         q = f(x1, *args1)
@@ -915,10 +926,15 @@ def rs_nth_root(p, n, x, prec):
     Multivariate series expansion of the nth root of ``p``.
 
     Parameters
-    ----------
-    n : `p**(1/n)` is returned.
-    x : :class:`PolyElement`
-    prec : Order of the expanded series.
+    ==========
+
+    p : Expr
+        The polynomial to computer the root of.
+    n : integer
+        The order of the root to be computed.
+    x : :class:`~.PolyElement`
+    prec : integer
+        Order of the expanded series.
 
     Notes
     =====
@@ -941,8 +957,6 @@ def rs_nth_root(p, n, x, prec):
     >>> rs_nth_root(3 + x + x*y, 3, x, 2)
     0.160249952256379*x*y + 0.160249952256379*x + 1.44224957030741
     """
-    p0 = p
-    n0 = n
     if n == 0:
         if p == 0:
             raise ValueError('0**0 expression')
@@ -951,7 +965,6 @@ def rs_nth_root(p, n, x, prec):
     if n == 1:
         return rs_trunc(p, x, prec)
     R = p.ring
-    zm = R.zero_monom
     index = R.gens.index(x)
     m = min(p, key=lambda k: k[index])[index]
     p = mul_xin(p, index, -m)
@@ -1138,7 +1151,6 @@ def rs_exp(p, x, prec):
         return _exp1(p, x, prec)
     one = R(1)
     n = 1
-    k = 1
     c = []
     for k in range(prec):
         c.append(one/n)
@@ -1265,16 +1277,16 @@ def rs_asin(p, x, prec):
 
 def _tan1(p, x, prec):
     r"""
-    Helper function of `rs\_tan`.
+    Helper function of :func:`rs_tan`.
 
     Return the series expansion of tan of a univariate series using Newton's
     method. It takes advantage of the fact that series expansion of atan is
     easier than that of tan.
 
-    Consider `f(x) = y - atan(x)`
+    Consider `f(x) = y - \arctan(x)`
     Let r be a root of f(x) found using Newton's method.
     Then `f(r) = 0`
-    Or `y  = atan(x)` where `x = tan(y)` as required.
+    Or `y = \arctan(x)` where `x = \tan(y)` as required.
     """
     R = p.ring
     p1 = R(0)
@@ -1480,11 +1492,11 @@ def rs_cos(p, x, prec):
     if c:
         if R.domain is EX:
             c_expr = c.as_expr()
-            t1, t2 = sin(c_expr), cos(c_expr)
+            _, _ = sin(c_expr), cos(c_expr)
         elif isinstance(c, PolyElement):
             try:
                 c_expr = c.as_expr()
-                t1, t2 = R(sin(c_expr)), R(cos(c_expr))
+                _, _ = R(sin(c_expr)), R(cos(c_expr))
             except ValueError:
                 R = R.add_gens([sin(c_expr), cos(c_expr)])
                 p = p.set_ring(R)
@@ -1492,7 +1504,7 @@ def rs_cos(p, x, prec):
                 c = c.set_ring(R)
         else:
             try:
-                t1, t2 = R(sin(c)), R(cos(c))
+                _, _ = R(sin(c)), R(cos(c))
             except ValueError:
                 raise DomainError("The given series can't be expanded in "
                     "this domain.")
@@ -1525,7 +1537,7 @@ def rs_cos(p, x, prec):
 
 def rs_cos_sin(p, x, prec):
     r"""
-    Return the tuple `(rs\_cos(p, x, prec)`, `rs\_sin(p, x, prec))`.
+    Return the tuple ``(rs_cos(p, x, prec)`, `rs_sin(p, x, prec))``.
 
     Is faster than calling rs_cos and rs_sin separately
     """
@@ -1662,7 +1674,7 @@ def rs_cosh(p, x, prec):
 
 def _tanh(p, x, prec):
     r"""
-    Helper function of `rs\_tanh`
+    Helper function of :func:`rs_tanh`
 
     Return the series expansion of tanh of a univariate series using Newton's
     method. It takes advantage of the fact that series expansion of atanh is
@@ -1806,10 +1818,10 @@ def rs_compose_add(p1, p2):
     References
     ==========
 
-    A. Bostan, P. Flajolet, B. Salvy and E. Schost
-    "Fast Computation with Two Algebraic Numbers",
-    (2002) Research Report 4579, Institut
-    National de Recherche en Informatique et en Automatique
+    .. [1] A. Bostan, P. Flajolet, B. Salvy and E. Schost
+           "Fast Computation with Two Algebraic Numbers",
+           (2002) Research Report 4579, Institut
+           National de Recherche en Informatique et en Automatique
     """
     R = p1.ring
     x = R.gens[0]
@@ -1948,7 +1960,8 @@ def rs_series(expr, a, prec):
     """Return the series expansion of an expression about 0.
 
     Parameters
-    ----------
+    ==========
+
     expr : :class:`Expr`
     a : :class:`Symbol` with respect to which expr is to be expanded
     prec : order of the series expansion
