@@ -1,7 +1,9 @@
 from sympy import Symbol, Contains, S, Interval, FiniteSet, oo, Eq
-
+from sympy.core.expr import unchanged
+from sympy.utilities.pytest import raises
 
 def test_contains_basic():
+    raises(TypeError, lambda: Contains(S.Integers, 1))
     assert Contains(2, S.Integers) is S.true
     assert Contains(-2, S.Naturals) is S.false
 
@@ -11,8 +13,8 @@ def test_contains_basic():
 
 def test_issue_6194():
     x = Symbol('x')
-    assert Contains(x, Interval(0, 1)) != (x >= 0) & (x <= 1)
-    assert Interval(0, 1).contains(x) == (x >= 0) & (x <= 1)
+    assert unchanged(Contains, x, Interval(0, 1))
+    assert Interval(0, 1).contains(x) == (S.Zero <= x) & (x <= 1)
     assert Contains(x, FiniteSet(0)) != S.false
     assert Contains(x, Interval(1, 1)) != S.false
     assert Contains(x, S.Integers) != S.false
@@ -34,5 +36,7 @@ def test_binary_symbols():
 def test_as_set():
     x = Symbol('x')
     y = Symbol('y')
-    assert Contains(x, FiniteSet(y)
-        ).as_set() == Contains(x, FiniteSet(y))
+    # Contains is a BooleanFunction whose value depends on an arg's
+    # containment in a Set -- rewriting as a Set is not yet implemented
+    raises(NotImplementedError, lambda:
+           Contains(x, FiniteSet(y)).as_set())
